import axios from "axios";
import {collectionExists, createCollection, putItem} from "@/store/backend";
import {DatabaseItem, NameKeyStore} from "@/store/interfaces/NameKeyStore";


export default class BilliDB {
    DB_VERSION: number;
    DB_NAME: string;
    BACKEND_URL: string;
    constructor(dbName:string, dbVersion:number, backendUrl:string) {
        this.DB_NAME = dbName;
        this.DB_VERSION = dbVersion;
        this.BACKEND_URL = backendUrl;
    }


    async initDb(stores: NameKeyStore[]):Promise<IDBOpenDBRequest> {
        return new Promise((resolve, reject) => {

            const request = window.indexedDB.open(this.DB_NAME, this.DB_VERSION);
            request.onerror = e => {
                console.log("Error opening db", e);
                reject("Error");
            }

            request.onsuccess =e => {
                console.log("connect erfolgreich");
                // @ts-ignore
                resolve(e.target.result)
            }
            request.onupgradeneeded = e => {
                console.log("onupgradeneeded");
                // @ts-ignore
                const db = e.target.result;

                stores.forEach((store) => {
                    db.createObjectStore(store.id, {autoIncrement:false, keyPath: store.identifier});
                })
            }
        })
    }


    async getDb():Promise<IDBOpenDBRequest> {
        return new Promise((resolve, reject) => {

            const request = window.indexedDB.open(this.DB_NAME, this.DB_VERSION);

            request.onerror = e => {
                console.log("Error opening db", e);
                reject("Error");
            }

            request.onsuccess =e => {
                console.log("connect erfolgreich");
                // @ts-ignore
                resolve(e.target.result)
            }

        })
    }


    async saveItem(item, storeId:string) {
        console.log("-->saveItem", storeId, item);
        const itemClean = {...item}
        console.log("-->saveItem clean", storeId, itemClean);

        const request = window.indexedDB.open(this.DB_NAME, this.DB_VERSION);
        request.onsuccess = (event) => {
            console.log("DB Request done");
            const transaction = request.result
                .transaction([storeId], 'readwrite');
            const objectStore = transaction.objectStore(storeId);

            const requestUpdate = objectStore.put(itemClean);
            requestUpdate.onsuccess = (event) => {
                console.log("Data update complete")
            }

            transaction.oncomplete = (event) => {
                console.log("Transaction complete")
            }
        }

        request.onerror = (err) => {
            console.log("DB Error" , err)
        }


/**
        return new Promise((resolve, reject) => {
            console.log("Starting Add Transaction");

            // @ts-ignore
            const trans = db.transaction([storeId], 'readwrite');

            trans.oncomplete = (e) => {
                resolve(e);
            };

            trans.onerror = (e) => {
               console.log("on Error", e)
                reject(e);
            };
            const store = trans.objectStore(storeId);
            console.log("ITEM.STRING", JSON.stringify(item.getDBVersion()));
            store.put(item.getDBVersion());

            store.onerror = (e) => {
                console.log("Error on Store", e)
            }
            store.oncomplete = (e) => {
                console.log("oncomplete on Store", e)
            }
            store.onsuccess = (e) => {
                console.log("onsuccess on Store", e)
            }
        })
            */
    }
    async saveItems(items, storeId:string) {
        const db = await this.getDb();

        return new Promise((resolve, reject) => {
            console.log("Starting Add Transaction");

            // @ts-ignore
            const trans = db.transaction([storeId], 'readwrite');

            trans.oncomplete = (e) => {
                resolve(e);
            };

            trans.onerror = (e) => {
                console.log("on Error", e)
                reject(e);
            };
            const store = trans.objectStore(storeId);

            items.forEach((item) => {
                store.add(item);
            })

            store.onerror = (e) => {
                console.log("Error on Store", e)
            }
            store.oncomplete = (e) => {
                console.log("oncomplete on Store", e)
            }
            store.onsuccess = (e) => {
                console.log("onsuccess on Store", e)
            }
        })
    }
    async getItems(storeId:string) {
        const db = await this.getDb();

        return new Promise((resolve, reject) => {
            const items = [];
            // @ts-ignore
            const trans = db.transaction([storeId], 'readonly');

            trans.oncomplete = (_) => {
                console.log(`${storeId}: oncomplete`, items.length)

                resolve(items);
            };

            trans.onerror = (e) => {
                console.log("on Error", e)
                reject(e);
            };
            const store = trans.objectStore(storeId);

            store.openCursor().onsuccess = (e) => {
                const cursor = e.target.result;
                if(cursor) {

                    items.push(cursor.value);
                    cursor.continue();
                }
            }

            store.onerror = (e) => {
                console.log("Error on getAll", e)
            }
            store.oncomplete = (e) => {
                console.log("oncomplete on getAll", e)
            }
        })
    }

    async syncDatabase(storeId:string) {
        console.log("-->syncDatabase");
        let db;
        const request = window.indexedDB.open(this.DB_NAME, this.DB_VERSION);
        request.onsuccess = (event) => {
            if (navigator.onLine) {
                db = request.result;
                console.log("..onsuccess", db)

                for(const collection of db.objectStoreNames) {
                    console.log("working on: ", collection);

                    createCollection(collection).then( exist => {
                        console.log(exist);
                        const transaction = db.transaction(collection,'readwrite');
                        const objectStore = transaction.objectStore(collection);
                        console.log("working on objectstore: ", objectStore);
                        const cursorRequest = objectStore.openCursor();
                        cursorRequest.onsuccess = (e) => {
                            const cursor = e.target.result;
                            if(cursor) {
                                const value = cursor.value;
                                console.log(cursor.key, cursor.value, value);

                                putItem(collection, value);
                                cursor.continue();
                            } else {
                                console.log("no cursor");
                            }
                        }
                        cursorRequest.onerror = (err) => {
                            console.log("cursor error", err);
                        }
                    }).catch(err => {
                        console.log(err);
                    });


                }
                /**
                const objectStore = transaction.objectStore('brand111s');
                const cursorRequest = objectStore.openCursor();

                cursorRequest.onsuccess = (e) => {
                    const cursor = e.target.result;
                    if (cursor) {

                        console.log("Cursor: ", cursor.value());

                        //                 axios.put('', cursor.value())
                        // Perform synchronization logic here
                        cursor.continue();
                    }
                }
                    */

            }
        }
        // Test if we have connection to the Internet


    }

}




