import { createApp } from 'vue'
import App from './App.vue'
import store from './store';
import router from './router'
import VueKeyCloak from '@dsb-norge/vue-keycloak-js'
import 'bootstrap/dist/css/bootstrap.css'

import { KeycloakInstance } from "keycloak-js";
import { VueKeycloakInstance } from "@dsb-norge/vue-keycloak-js/dist/types";

import {library} from "@fortawesome/fontawesome-svg-core";
import { faUserSecret,faTelescope,
    faCamera,
    faAperture,
    faBoxFull,
    faTag,
    faToggleOff,
    faPlus,
    faXmark,
    faPen,
    faFloppyDisk,
    faToggleOn } from "@fortawesome/pro-solid-svg-icons";
import { FontAwesomeIcon } from "@fortawesome/vue-fontawesome";

library.add(faUserSecret);

const SKIP_KEYCLOAK = false;


//
//
library.add(faTelescope);
library.add(faCamera);
library.add(faAperture);
library.add(faBoxFull);
library.add(faTag);
library.add(faToggleOff);
library.add(faToggleOn);
library.add(faPlus);
library.add(faXmark);
library.add(faPen);
library.add(faFloppyDisk);

const app =
createApp(App);



app.use(store)
app.use(router)
app.component("font-awesome-icon", FontAwesomeIcon);
if (!SKIP_KEYCLOAK) app.use(VueKeyCloak, {
        config: {
            url: 'https://auth.toking.de/',
            realm: 'toking',
            clientId: 'billibox',
            redirectUri: 'http://127.0.0.1:8081/',
            onLoad: 'login-required',
            enableLogging: true,
            scope: 'open-id'
        },
        init: {
            onLoad: 'login-required'
        },
        onReady (keycloak: KeycloakInstance) {
           store.dispatch("storeKeycloak", keycloak).then( (res) => {

                   keycloak.loadUserProfile().then( (res) => {
                       store.dispatch("storeUserProfile", res);
                       console.log("res", res);
                   });

                   console.log(keycloak.idToken);

           }
           )
        }
    })

app.mount('#app')


declare module '@vue/runtime-core' {
    interface ComponentCustomProperties  {
        $keycloak: VueKeycloakInstance
    }
}
