import axios from "axios";
import {backendAPI, indexDB} from "@/config";
import Util from "@/util";
import {DatabaseItem} from "@/store/interfaces/NameKeyStore";


export async function collectionExists(collectionName: string) {
    const apiBase = Util.getURL(backendAPI);
    const endpoint = `${apiBase}/pgadmin/${indexDB.id}/${collectionName}`;
    try {
        const rawResult = await axios.get(endpoint);
        return true;
    } catch (e) {
        return false;
    }
}

export async function createCollection(collectionName: string) {
    const apiBase = Util.getURL(backendAPI);
    const endpoint = `${apiBase}/pgadmin/${indexDB.id}/${collectionName}`;
    try {
        const rawResult = await axios.post(endpoint);
        return true;
    } catch (e) {
        return false;
    }
}

export async function putItem(collectionName: string, item:DatabaseItem) {
    console.log("-->putItem", item);
    const apiBase = Util.getURL(backendAPI);
    const endpoint = `${apiBase}/pg/${indexDB.id}/${collectionName}/${item.key}`;
    try {
        const rawResult = await axios.put(endpoint,item);
        return rawResult.data;
    } catch (e) {
        return false;
    }
}