import InventoryItem from "@/store/classes/InventoryItem";

export default class Camera extends InventoryItem {

    private ADD_FIELDS = ['light_measure_method', 'mount_key',
        'buildtype_key', 'media_key', 'year_manufactured_from', 'year_manufactured_to']

    private _light_measure_method: string;
    private _mount_key: string;
    private _buildtype_key: string;
    private _media_key: string;


    private _year_manufactured_from: string;
    private _year_manufactured_to: string;

    constructor(id:string, name:string, values?:Object) {
        super(id, name, values);
        this._item_type = "camera";

        if(typeof values != "undefined") {
            for (const field  of this.ADD_FIELDS) {
                if(field in values) this["_" + field] = values[field];
            }
        }
    }

    getDBVersion():object {
        const obj = {
            key: this.key,
            item_type: this._item_type,
            manufacturer_key: this.manufacturer_key,
            brand_key: this.brand_key,
            condition_key: this.condition_key,
            name: this.name,
            description: this.description,
            work_done: this.work_done,
            special: this.special,
            price_purchased: this.price_purchased,
            price_historic: this.price_historic,
            price_current: this.price_current,
            year_of_production: this.year_of_production,
            year_of_purchase: this.year_of_purchase,
            inventory_number: this.inventory_number,
            set_key: this.set_key,
            light_measure_method: this._light_measure_method,
            mount_key: this._mount_key,
            buildtype_key: this._buildtype_key,
            media_key: this._media_key,
            year_manufactured_from: this._year_manufactured_from,
            year_manufactured_to: this._year_manufactured_to,


        }
        return obj
    }


    get light_measure_method(): string {
        return this._light_measure_method;
    }

    get mount_key(): string {
        return this._mount_key;
    }

    get buildtype_key(): string {
        return this._buildtype_key;
    }

    get media_key(): string {
        return this._media_key;
    }

    get year_manufactured_from(): string {
        return this._year_manufactured_from;
    }

    get year_manufactured_to(): string {
        return this._year_manufactured_to;
    }


    set light_measure_method(value: string) {
        this._light_measure_method = value;
    }

    set mount_key(value: string) {
        this._mount_key = value;
    }

    set buildtype_key(value: string) {
        this._buildtype_key = value;
    }

    set media_key(value: string) {
        this._media_key = value;
    }

    set year_manufactured_from(value: string) {
        this._year_manufactured_from = value;
    }

    set year_manufactured_to(value: string) {
        this._year_manufactured_to = value;
    }
}