import {date} from "yup";


export default class InventoryItem {

    private FIELDS = ['item_type', 'manufacturer_key', 'brand_key', 'condition_key',
        'name', 'description', 'work_done', 'special',
        'price_purchased', 'price_historic', 'price_current',
        'year_of_production', 'year_of_purchase', 'inventory_number', 'set_key' ];

    private _key: string;
    private _inventory_number: string;

    protected _item_type: "camera" | "accessoire" | "lens" | "set";

    private _active: boolean;
    private _edit: boolean;

    private _manufacturer_key: string;
    private _brand_key: string;
    private _condition_key: string;

    private _name: string;
    private _description: string;
    private _work_done: string;
    private _special: string;

    private _price_purchased: number;
    private _price_historic: number;
    private _price_current: number;

    private _year_of_production: number;
    private _year_of_purchase: number;

    private _set_key: string;

    private _created: Date;
    private _changed: Date;



    constructor( key:string, name:string, values?:Object ) {
        this._key = key;
        this._name = name;
        this._active = false;
        this._edit = false;
        this._created = new Date();
        this._changed = new Date();

        if(typeof values != "undefined") {
            for (const field  of this.FIELDS) {
                if(field in values) this["_" + field] = values[field];
            }
        }
    }

    getDBVersion():object {
        const obj = {
            key: this._key,
            item_type: this._item_type,
            manufacturer_key: this._manufacturer_key,
            brand_key: this._brand_key,
            condition_key: this._condition_key,
            name: this._name,
            description: this._description,
            work_done: this._work_done,
            special: this._special,
            price_purchased: this._price_purchased,
            price_historic: this._price_historic,
            price_current: this._price_current,
            year_of_production: this._year_of_production,
            year_of_purchase: this._year_of_purchase,
            inventory_number: this._inventory_number,
            set_key: this._set_key
        }
        return obj
    }


    get key(): string {
        return this._key;
    }

    get active(): boolean {
        return this._active;
    }

    get edit(): boolean {
        return this._edit;
    }

    get item_type(): "camera" | "accessoire" | "lens" | "set" {
        return this._item_type;
    }

    get manufacturer_key(): string {
        return this._manufacturer_key;
    }

    get brand_key(): string {
        return this._brand_key;
    }

    get condition_key(): string {
        return this._condition_key;
    }

    get name(): string {
        return this._name;
    }

    get description(): string {
        return this._description;
    }

    get work_done(): string {
        return this._work_done;
    }

    get special(): string {
        return this._special;
    }

    get price_purchased(): number {
        return this._price_purchased;
    }

    get price_historic(): number {
        return this._price_historic;
    }

    get price_current(): number {
        return this._price_current;
    }

    get year_of_production(): number {
        return this._year_of_production;
    }

    get year_of_purchase(): number {
        return this._year_of_purchase;
    }


    set active(value: boolean) {
        this._active = value;
    }

    set edit(value: boolean) {
        this._edit = value;
    }

    set manufacturer_key(value: string) {
        this._manufacturer_key = value;
    }

    set brand_key(value: string) {
        this._brand_key = value;
    }

    set condition_key(value: string) {
        this._condition_key = value;
    }

    set name(value: string) {
        this._name = value;
    }

    set description(value: string) {
        this._description = value;
    }

    set work_done(value: string) {
        this._work_done = value;
    }

    set special(value: string) {
        this._special = value;
    }

    set price_purchased(value: number) {
        this._price_purchased = Number(value);
    }

    set price_historic(value: number) {
        this._price_historic = Number(value);
    }

    set price_current(value: number) {
        this._price_current = Number(value);
    }

    set year_of_production(value: number) {
        this._year_of_production = value;
    }

    set year_of_purchase(value: number) {
        this._year_of_purchase = value;
    }


    get set_key(): string {
        return this._set_key;
    }

    set set_key(value: string) {
        this._set_key = value;
    }


    get inventory_number(): string {
        return this._inventory_number;
    }

    get created(): Date {
        return this._created;
    }

    get changed(): Date {
        return this._changed;
    }

    set changed(value: Date) {
        this._changed = value;
    }
}

