import InventoryItem from "@/store/classes/InventoryItem";

export default class Lens extends InventoryItem {

    private ADD_FIELDS = ['mount_key', 'filtermount_key', 'autofocus', 'aperture_max', 'focal_length_min', 'focal_length_max' ];

    private _mount_key: string;
    private _autofocus: boolean;
    private _aperture_max: number;
    private _focal_length_min: number;
    private _focal_length_max: number;
    private _filtermount_key: string;

    constructor(id:string, name:string, values?:Object) {
        super(id, name, values);
        this._item_type = "lens";

            if(typeof values != "undefined") {
                for (const field  of this.ADD_FIELDS) {
                    if(field in values) this["_" + field] = values[field];
                }
            }
    }
    getDBVersion():object {
        const obj = {
            key: this.key,
            item_type: this._item_type,
            manufacturer_key: this.manufacturer_key,
            brand_key: this.brand_key,
            condition_key: this.condition_key,
            name: this.name,
            description: this.description,
            work_done: this.work_done,
            special: this.special,
            price_purchased: this.price_purchased,
            price_historic: this.price_historic,
            price_current: this.price_current,
            year_of_production: this.year_of_production,
            year_of_purchase: this.year_of_purchase,
            inventory_number: this.inventory_number,
            set_key: this.set_key,
            mount_key: this._mount_key,
            filtermount_key: this._filtermount_key,
            autofocus: this._autofocus,
            aperture_max: this._aperture_max,
            focal_length_min: this._focal_length_min,
            focal_length_max: this._focal_length_max,

        }
        return obj
    }

    get mount_key(): string {
        return this._mount_key;
    }

    get autofocus(): boolean {
        return this._autofocus;
    }

    get aperture_max(): number {
        return this._aperture_max;
    }

    get focal_length_min(): number {
        return this._focal_length_min;
    }

    get focal_length_max(): number {
        return this._focal_length_max;
    }

    get filtermount_key(): string {
        return this._filtermount_key;
    }


    set mount_key(value: string) {
        this._mount_key = value;
    }

    set autofocus(value: boolean) {
        this._autofocus = value;
    }

    set aperture_max(value: number) {
        this._aperture_max = value;
    }

    set focal_length_min(value: number) {
        this._focal_length_min = value;
    }

    set focal_length_max(value: number) {
        this._focal_length_max = value;
    }

    set filtermount_key(value: string) {
        this._filtermount_key = value;
    }
}


const camera2 = new Lens(null, "Pentax K30", {
    manufacturer_key: "manufacturere",
    brand_key: "brand",
    condition_key: "condition",
    buildtype_key: "buildtype",
    autofocus: true,
});
console.log(camera2);
