import * as seed from "@/seed";
import BilliDB from "@/indexdDB";
import { v4 as uuidv4 } from 'uuid';
import { SEED_VERSION } from "@/seed";
import InventoryItem from "@/store/classes/InventoryItem";
import Camera from "@/store/classes/Camera";
import Lens from "@/store/classes/Lens";
import InventorySet from "@/store/classes/InventorySet";
const db = new BilliDB("billibox", 3);
const stores = [
    { id: "inventory", identifier: "key", name: 'Inventar', type: 'object' },
    { id: "brands", identifier: "key", name: "Marken", type: "nameKey" },
    { id: "conditions", identifier: "key", name: "Zustände", type: "nameKey" },
    { id: "buildtypes", identifier: "key", name: "Bauformen", type: "nameKey" },
    { id: "manufacturers", identifier: "key", name: "Hersteller", type: "nameKey" },
    { id: "medias", identifier: "key", name: "Medien", type: "nameKey" },
    { id: "mounts", identifier: "key", name: "Anschlüsse", type: "nameKey" },
    { id: "filtermounts", identifier: "key", name: "Filteranschlüsse", type: "nameKey" },
    { id: "assets", identifier: "key", name: "Dateien", type: "filestore" },
];
const state = {
    isInitialized: false,
    inventory: [],
    assets: [],
    brands: [],
    sets: [],
    manufacturers: [],
    conditions: [],
    medias: [],
    mounts: [],
    buildtypes: [],
    filtermounts: []
};
const getters = {
    inventory: (state) => {
        return state.inventory;
    },
    stores: (state) => stores,
    store: (state) => (key) => {
        console.log("Query store:", key);
        return state[key];
    },
    cameras: (state) => {
        return state.inventory.find((item) => item.item_type === "camera");
    },
    camera: (state) => (key) => state.inventory.find((item) => item.item_type === "camera" && item.key === key),
    activeItem: (state) => state.inventory.find((item) => item.active),
    lenses: (state) => {
        return state.inventory.find((item) => item.item_type === "lens");
    },
    lens: (state) => (key) => state.inventory.find((item) => item.item_type === "lens" && item.key === key),
    accessories: (state) => {
        return state.inventory.find((item) => item.item_type === "accessoire");
    },
    accessoire: (state) => (key) => state.inventory.find((item) => item.item_type === "accessoire" && item.key === key),
    brands: (state) => state.brands,
    brand: (state) => (key) => state.brands.find((brand) => brand.key === key),
    manufacturers: (state) => state.manufacturers,
    manufacturer: (state) => (key) => state.manufacturers.find((item) => item.key === key),
    mounts: (state) => state.mounts,
    mount: (state) => (key) => state.mounts.find((item) => item.key === key),
    filtermounts: (state) => state.filtermounts,
    filtermount: (state) => (key) => state.filtermounts.find((item) => item.key === key),
    conditions: (state) => state.conditions,
    condition: (state) => (key) => state.conditions.find((condition) => condition.key === key),
    sets: (state) => state.inventory.find((item) => item.item_type === "set"),
    set: (state) => (key) => state.inventory.find((item) => item.key === key),
    medias: (state) => state.medias,
    media: (state) => (key) => state.medias.find((media) => media.key === key),
    buildtypes: (state) => state.buildtypes,
    buildtype: (state) => (key) => state.buildtypes.find((buildtype) => buildtype.key === key),
    isInitialized: (state) => state.isInitialized,
    getIconName: (state) => (item_type) => {
        switch (item_type) {
            case "camera":
                return "camera";
            case "set":
                return "box-open-full";
            case "accessoire":
                return "telescope";
            case "lens":
                return "aperture";
        }
        return "telescope";
    }
};
const mutations = {
    setInventory(state, payload) {
        console.log("mutation.setInventory");
        payload.map((item) => {
            let newInventoryItem;
            switch (item.item_type) {
                case "set":
                    newInventoryItem = new InventorySet(item.key, item.name, item);
                    break;
                case "camera":
                    newInventoryItem = new Camera(item.key, item.name, item);
                    break;
                case "lens":
                    newInventoryItem = new Lens(item.key, item.name, item);
                    break;
                default:
                    newInventoryItem = new InventoryItem(item.key, item.name, item);
            }
            state.inventory.push(newInventoryItem);
        });
        console.log("STORE", state.inventory.length);
    },
    storeCamera(sate, payload) {
        const objCamera = state.inventory.find((camera) => camera.key === payload.key);
        if (objCamera) {
            payload.edit = false;
            objCamera.name = payload.name;
        }
        else {
            payload.edit = false;
            state.inventory.push(payload);
        }
        return payload.key;
    },
    storeNameKey(state, payload) {
        const objNK = state[payload.storeType].find((item) => item.key === payload.itemOld.key);
        if (objNK.key)
            objNK.key = payload.item.key;
        objNK.name = payload.item.name;
        objNK.description = payload.item.description;
    },
    insertNameKey(state, payload) {
        const objNK = state[payload.storeType].find((item) => item.key === payload.item.key);
        if (objNK)
            throw new Error("Key already exists");
        state[payload.storeType].push(payload.item);
    },
    setItemActiveState(state, payload) {
        console.log("mutation.setItemActiveState");
        state.inventory.map((item) => item.active = false);
        const objItem = state.inventory.find((itemId) => itemId.key === payload.key);
        objItem.active = payload.active;
        console.log("Result of mutation", objItem);
    },
    setItemEditState(state, payload) {
        state.inventory.map((item) => item.edit = false);
        const objItem = state.inventory.find((camera) => camera.key === payload.key);
        objItem.edit = payload.edit;
    },
    setNameKeys(state, payload) {
        const field = payload.namekey_type;
        state[field] = payload.values;
    },
    setInitialized(state, payload) {
        state.isInitialized = payload;
    },
    storeInventoryDescription(state, payload) {
        const objItem = state.inventory.find((item) => item.key === payload.key);
        objItem.description = payload.description;
        objItem.changed = new Date();
        db.saveItem(objItem, "inventory").then((res) => {
            console.log("DB-Save resolved", res);
        }).catch((e) => console.log(e));
    },
    storeInventoryLensDetail(state, payload) {
        console.log("storeInventoryLensDetail", payload);
        const objItem = state.inventory.find((item) => item.key === payload.key);
        objItem.focal_length_max = payload.item.focal_length_max;
        objItem.focal_length_min = payload.item.focal_length_min;
        objItem.autofocus = payload.item.autofocus;
        objItem.year_of_production = payload.item.year_of_production;
        objItem.aperture_max = payload.item.aperture_max;
        objItem.changed = new Date();
        db.saveItem(objItem, "inventory").then((res) => {
            console.log("DB-Save resolved", res);
        }).catch((e) => console.log(e));
    },
    storeInventoryCollectionInfo(state, payload) {
        const objItem = state.inventory.find((item) => item.key === payload.key);
        objItem.year_of_purchase = payload.item.year_of_purchase;
        objItem.condition_key = payload.item.condition_key;
        objItem.price_current = payload.item.price_current;
        objItem.price_historic = payload.item.price_historic;
        objItem.price_purchased = payload.item.price_purchased;
        objItem.changed = new Date();
        db.saveItem(objItem, "inventory").then((res) => {
            console.log("DB-Save resolved", res);
        }).catch((e) => console.log(e));
    },
    storeInventoryCameraDetails(state, payload) {
        const objItem = state.inventory.find((item) => item.key === payload.key);
        objItem.brand_key = payload.item.brand_key;
        objItem.manufacturer_key = payload.item.manufacturer_key;
        objItem.mount_key = payload.item.mount_key;
        objItem.buildtype_key = payload.item.buildtype_key;
        objItem.media_key = payload.item.media_key;
        objItem.year_of_production = payload.item.year_of_production;
        objItem.year_manufactured_from = payload.item.year_manufactured_from;
        objItem.year_manufactured_to = payload.item.year_manufactured_to;
        objItem.light_measure_method = payload.item.light_measure_method;
        objItem.changed = new Date();
        db.saveItem(objItem, "inventory").then((res) => {
            console.log("DB-Save resolved", res);
        }).catch((e) => console.log(e));
    }
};
const actions = {
    storeNameKeyObject(context, payload) {
        console.log("actions.storeNameKeyObject", payload);

        db.saveItem(payload, payload.storeType).then((res) => {
            console.log("Item stored in Database");
        });
        context.commit("storeNameKey", {...payload});
    },
    insertNameKeyObject(context, payload) {
        console.log("actions.insertNameKeyObject", payload);
        db.saveItem(payload.item, payload.storeType).then((res) => {
            console.log("Item stored in Database");
        });
        context.commit("insertNameKey", payload);
    },
    fetchNameKeys(context, payload) {
        const field = payload.namekey_type;
        db.getItems(field).then((items) => {
            context.commit("setNameKeys", {
                namekey_type: field,
                values: items
            });
        });
    },
    fetchInventory(context, payload) {
        console.log("action.fetchInventory");
        db.getItems("inventory").then((items) => {
            context.commit("setInventory", items);
        });
    },
    setItemActiveState(context, payload) {
        context.commit("setItemActiveState", { key: payload.key, active: payload.active });
    },
    setItemEditState(context, payload) {
        console.log("action.setItemEditState", payload);
        context.commit("setItemEditState", { key: payload.key, edit: payload.edit });
    },
    storeCamera(context, payload) {
        // console.log("action.storeCamera", payload)
        if (!payload.key)
            payload.key = uuidv4();
        //console.log(" ... with key ", payload.key);
        //console.log("Try to save to indexDB");
        db.saveItem(payload, "inventory").then((res) => {
            console.log("DB-Save resolved", res);
        }).catch((e) => console.log(e));
        context.commit("storeCamera", payload);
        context.commit("setCameraActiveState", { cameraId: payload.key, active: true });
    },
    storeInventoryDescription(context, payload) {
        context.commit("storeInventoryDescription", payload);
    },
    storeInventoryLensDetail(context, payload) {
        context.commit("storeInventoryLensDetail", payload);
    },
    storeInventoryCollectionInfo(context, payload) {
        context.commit("storeInventoryCollectionInfo", payload);
    },
    storeInventoryCameraDetails(context, payload) {
        context.commit("storeInventoryCameraDetails", payload);
    },
    loadDataFromLocalDB(context) {
        console.log("action.loadDataFromLocalDB");
        const storeActions = [];
        for (const store of stores) {
            if (store.type == "object") {
                storeActions.push(context.dispatch("fetchInventory"));
            }
            else {
                storeActions.push(context.dispatch("fetchNameKeys", { namekey_type: store.id }));
            }
        }
        Promise.all(storeActions).then(() => {
            context.commit("setInitialized", true);
        });
    },
    initialize(context, payload) {
        db.initDb(stores).then(() => {
            const promises = [];
            const seedVersion = localStorage.getItem("SEED_VERSION") || 0;
            if (payload.seed && seedVersion < SEED_VERSION) {
                console.log("SEEDING database");
                for (const store of stores) {
                    promises.push(db.saveItems(seed[store.id], store.id));
                }
                localStorage.setItem("SEED_VERSION", seed.SEED_VERSION.toString());
            }
            else {
                console.log("SEEDING skipped");
                db.syncDatabase();
                context.dispatch("loadDataFromLocalDB")
                    .then(() => console.log("Store Data loaded"))
                    .catch((e) => console.log("Error loading StoreData", e));
            }
        });
    }
};
const modules = {};
const inventoryModule = {
    state,
    getters,
    mutations,
    actions,
    modules
};
export default inventoryModule;
//# sourceMappingURL=index.js.map