import {ActionContext} from "vuex";
import * as seed from "@/seed";
import BilliDB from "@/indexdDB";
import { v4 as uuidv4 } from 'uuid';
import {SEED_VERSION} from "@/seed";
import NameKey from "@/store/classes/NameKey";
import InventoryItem from "@/store/classes/InventoryItem";
import Camera from "@/store/classes/Camera";
import Lens from "@/store/classes/Lens";
import InventorySet from "@/store/classes/InventorySet";
import {backendAPI, indexDB} from "@/config";
import Util from "@/util";
import {NameKeyStore} from "@/store/interfaces/NameKeyStore";


const db = new BilliDB(indexDB.id, 3, Util.getURL(backendAPI));


const stores:NameKeyStore[] = [
    {id: "inventory", identifier:"key", name: 'Inventar', type: 'object'},
    {id: "brands", identifier:"key", name: "Marken", type: "nameKey"},
    {id: "conditions", identifier:"key", name: "Zustände", type: "nameKey"},
    {id: "buildtypes", identifier:"key", name: "Bauformen", type: "nameKey"},
    {id: "manufacturers", identifier:"key", name: "Hersteller", type: "nameKey"},
    {id: "medias", identifier:"key", name: "Medien", type: "nameKey"},
    {id: "mounts", identifier:"key", name: "Anschlüsse", type: "nameKey"},
    {id: "filtermounts", identifier:"key", name: "Filteranschlüsse", type: "nameKey"},
    {id: "assets", identifier:"key", name: "Dateien", type: "filestore"},

];


const state = {
    isInitialized: false,
    inventory: [],
    assets:[],
    brands: [],
    sets: [],
    manufacturers: [],
    conditions: [],
    medias: [],
    mounts: [],
    buildtypes: [],
    filtermounts: []

};

const getters = {
    inventory: (state:any) => {
      return state.inventory
    },

    stores: (state?:any) => stores,
    store: (state:any) => (key: any) => {
        console.log("Query store:", key);
        return state[key]
    },

    cameras: (state: any) => {
       return state.inventory.find((item:InventoryItem) => item.item_type ==="camera");
    },
    camera: (state: any) => (key: any) => state.inventory.find((item: any) => item.item_type==="camera" &&item.key === key),
    activeItem: (state: any) => state.inventory.find((item:any) => item.active ),

    lenses: (state: any) => {
        return state.inventory.find((item:InventoryItem) => item.item_type ==="lens");
    },
    lens: (state: any) => (key: any) => state.inventory.find((item: any) => item.item_type==="lens" &&item.key === key),

    accessories: (state: any) => {
        return state.inventory.find((item:InventoryItem) => item.item_type ==="accessoire");
    },
    accessoire: (state: any) => (key: any) => state.inventory.find((item: any) => item.item_type==="accessoire" &&item.key === key),

    brands: (state: any) => state.brands,
    brand: (state: any) => (key: any) => state.brands.find((brand: any) => brand.key === key),

    manufacturers: (state: any) => state.manufacturers,
    manufacturer: (state: any) => (key: any) => state.manufacturers.find((item: any) => item.key === key),

    mounts: (state: any) => state.mounts,
    mount: (state: any) => (key: any) => state.mounts.find((item: any) => item.key === key),

    filtermounts: (state: any) => state.filtermounts,
    filtermount: (state: any) => (key: any) => state.filtermounts.find((item: any) => item.key === key),

    conditions: (state: any) => state.conditions,
    condition: (state: any) => (key: any) => state.conditions.find((condition: any) => condition.key === key),

    sets: (state: any) => state.inventory.find((item:InventoryItem) => item.item_type === "set" ),
    set: (state: any) => (key: any) => state.inventory.find((item: any) => item.key === key),

    medias: (state: any) => state.medias,
    media: (state: any) => (key: any) => state.medias.find((media: any) => media.key === key),

    buildtypes: (state: any) => state.buildtypes,
    buildtype: (state: any) => (key: any) => state.buildtypes.find((buildtype: any) => buildtype.key === key),

    isInitialized: (state:any ) => state.isInitialized,

    getIconName: (state:any) => (item_type) => {
        switch (item_type) {
            case "camera":
                return "camera";
            case "set":
                return "box-open-full";
            case "accessoire":
                return "telescope";
            case "lens":
                return "aperture";
        }
        return "telescope"
    }

};
const mutations = {
    setInventory(state:any, payload:any) {
        console.log("mutation.setInventory");
        payload.map((item:InventoryItem) => {
            let newInventoryItem:InventoryItem;

            switch(item.item_type) {
                case "set":
                    newInventoryItem = new InventorySet(item.key, item.name, item);
                    break;
                case "camera":
                    newInventoryItem = new Camera(item.key, item.name, item);
                    break;
                case "lens":
                    newInventoryItem = new Lens(item.key, item.name, item);
                    break;
                default:
                    newInventoryItem =  new InventoryItem(item.key, item.name, item);
            }
            state.inventory.push(newInventoryItem)
        })
        console.log("STORE", state.inventory.length);
    },

    storeCamera(sate:any, payload:any) {
        const objCamera = state.inventory.find((camera:Camera) => camera.key === payload.key);
        if(objCamera) {
            payload.edit =false;
            objCamera.name = payload.name;
        } else {
            payload.edit = false;
            state.inventory.push(payload);
        }
        return payload.key;
    },

    storeNameKey(state:any, payload:any) {
        console.log("mutation.storeNameKey", state, payload.storeType);
        const objNK:NameKey = state[payload.storeType].find( (item) => item.key === payload.item.key);
        if(objNK.key) objNK.key = payload.item.key;
        objNK.name = payload.item.name;
        objNK.description = payload.item.description;
    },

    insertNameKey(state:any, payload:{storeType:string, item:{key:string, name:string, description?:string}}) {
        const objNK:NameKey = state[payload.storeType].find( (item) => item.key === payload.item.key);
        if(objNK) throw new Error("Key already exists");
        state[payload.storeType].push(payload.item);
    },


    setItemActiveState(state:any, payload:any) {
        console.log("mutation.setItemActiveState");
        state.inventory.map((item:InventoryItem) => item.active= false);
        const objItem = state.inventory.find((itemId:InventoryItem) => itemId.key === payload.key);
        objItem.active = payload.active;
        console.log("Result of mutation", objItem);
    },

    setItemEditState(state:any, payload:any) {
        state.inventory.map((item:InventoryItem) => item.edit= false);
        const objItem = state.inventory.find((camera:any) => camera.key === payload.key);
        objItem.edit = payload.edit;
    },

    setNameKeys(state:any, payload:any) {
        const field = payload.namekey_type;
        state[field] = payload.values;
    },

    setInitialized(state:any, payload:boolean) {
        state.isInitialized = payload;
    },

    storeInventoryDescription(state:any, payload:any) {
        const objItem = <InventoryItem>state.inventory.find((item:InventoryItem) => item.key === payload.key);
        objItem.description = payload.description;
        objItem.changed = new Date();

        db.saveItem(objItem , "inventory").then( (res) => {
            console.log("DB-Save resolved", res )}
        ).catch((e)=> console.log(e));
    },
    storeInventoryLensDetail(state:any, payload:any) {
        console.log("storeInventoryLensDetail",payload);
        const objItem = <Lens>state.inventory.find((item:InventoryItem) => item.key === payload.key);
        objItem.focal_length_max = payload.item.focal_length_max;
        objItem.focal_length_min = payload.item.focal_length_min;
        objItem.autofocus = payload.item.autofocus;
        objItem.year_of_production = payload.item.year_of_production;
        objItem.aperture_max = payload.item.aperture_max;
        objItem.changed = new Date();

        db.saveItem(objItem , "inventory").then( (res) => {
            console.log("DB-Save resolved", res )}
        ).catch((e)=> console.log(e));

    },

    storeInventoryCollectionInfo(state:any, payload:any) {
        const objItem = <InventoryItem>state.inventory.find((item:InventoryItem) => item.key === payload.key);
        objItem.year_of_purchase = payload.item.year_of_purchase;
        objItem.condition_key = payload.item.condition_key;
        objItem.price_current = payload.item.price_current;
        objItem.price_historic = payload.item.price_historic;
        objItem.price_purchased = payload.item.price_purchased;
        objItem.changed = new Date();


        db.saveItem(objItem , "inventory").then( (res) => {
            console.log("DB-Save resolved", res )}
        ).catch((e)=> console.log(e));

    },

    storeInventoryCameraDetails(state:any, payload:any) {
        const objItem = <Camera>state.inventory.find((item:InventoryItem) => item.key === payload.key);

        objItem.brand_key = payload.item.brand_key;
        objItem.manufacturer_key = payload.item.manufacturer_key;
        objItem.mount_key = payload.item.mount_key;
        objItem.buildtype_key = payload.item.buildtype_key;
        objItem.media_key = payload.item.media_key;
        objItem.year_of_production = payload.item.year_of_production;
        objItem.year_manufactured_from = payload.item.year_manufactured_from;
        objItem.year_manufactured_to = payload.item.year_manufactured_to;
        objItem.light_measure_method = payload.item.light_measure_method;
        objItem.changed = new Date();

        db.saveItem(objItem , "inventory").then( (res) => {
            console.log("DB-Save resolved", res )}
        ).catch((e)=> console.log(e));

    }




};
const actions = {

    storeNameKeyObject(context:ActionContext<any, any>, payload:{storeType:string, item:object}) {
        console.log("actions.storeNameKeyObject", payload);

        db.saveItem(payload.item, payload.storeType).then((res) => {
            console.log("Item stored in Database")
        })

        context.commit("storeNameKey", {...payload});

    },


    insertNameKeyObject(context:ActionContext<any, any>, payload:{storeType:string, item:NameKey}) {
        console.log("actions.insertNameKeyObject", payload);

        db.saveItem(payload.item, payload.storeType).then((res) => {
            console.log("Item stored in Database")
        })

        context.commit("insertNameKey", payload);

    },


    fetchNameKeys(context:ActionContext<any, any>, payload) {
        const field =payload.namekey_type;
        db.getItems(field).then((items) => {
            context.commit("setNameKeys", {
                namekey_type: field,
                values:items
            });
        })
    },

    fetchInventory(context:ActionContext<any, any>, payload) {
        console.log("action.fetchInventory")
        db.getItems("inventory").then((items) => {
            context.commit("setInventory", items);
        })
    },

    setItemActiveState(context:ActionContext<any, any>, payload:any) {
        context.commit("setItemActiveState", {key: payload.key, active:payload.active});

    },

    setItemEditState(context:ActionContext<any, any>, payload:any) {
        console.log("action.setItemEditState", payload)
        context.commit("setItemEditState", {key: payload.key, edit:payload.edit});

    },

    storeCamera(context:ActionContext<any, any>, payload:any) {
       // console.log("action.storeCamera", payload)

        if(!payload.key) payload.key = uuidv4();
        //console.log(" ... with key ", payload.key);

        //console.log("Try to save to indexDB");
        db.saveItem(payload , "inventory").then( (res) => {
            console.log("DB-Save resolved", res )}
        ).catch((e)=> console.log(e));


        context.commit("storeCamera", payload);
        context.commit("setCameraActiveState", {cameraId:payload.key, active:true})

    },

    storeInventoryDescription(context:ActionContext<any, any>, payload:any) {
        context.commit("storeInventoryDescription", payload);
    },


    storeInventoryLensDetail(context:ActionContext<any, any>, payload:{key:string, item:Lens}) {
        context.commit("storeInventoryLensDetail", payload)
    },

    storeInventoryCollectionInfo(context:ActionContext<any, any>, payload:{key:string, item:InventoryItem}) {
        context.commit("storeInventoryCollectionInfo", payload);
    },

    storeInventoryCameraDetails(context:ActionContext<any, any>, payload:{key:string, item:Camera}) {
        context.commit("storeInventoryCameraDetails", payload);
    },


    loadDataFromLocalDB(context:ActionContext<any, any>) {
        console.log("action.loadDataFromLocalDB")
        const storeActions = [];
        for (const store of stores) {
            if(store.type =="object") {
                storeActions.push(context.dispatch("fetchInventory"));
            }
            else {
                storeActions.push(context.dispatch("fetchNameKeys", {namekey_type:store.id}));
            }
        }
        Promise.all(storeActions).then( () => {
            context.commit("setInitialized", true);
        });

    },

    initialize(context:ActionContext<any, any>, payload) {

        db.initDb(stores).then( () => {
            const promises = [];
            const seedVersion = localStorage.getItem("SEED_VERSION") || 0;
            if(payload.seed && seedVersion < SEED_VERSION ) {
                console.log("SEEDING database");
                for (const store of stores) {
                    promises.push(db.saveItems(seed[store.id], store.id));
                }
                localStorage.setItem("SEED_VERSION", seed.SEED_VERSION.toString())
            } else {
                console.log("SEEDING skipped");
                db.syncDatabase("test");
                context.dispatch("loadDataFromLocalDB")
                    .then(() => console.log("Store Data loaded"))
                    .catch((e) => console.log("Error loading StoreData", e));

            }
        });
    }
};

const modules = {};

const inventoryModule = {
    state,
    getters,
    mutations,
    actions,
    modules
}

export default inventoryModule